DECLARE SUB MdelayASM (Ziel&, asm$)
DECLARE SUB MDelay16 (Ziel&)
DECLARE SUB MDelay8 (Ziel%)

'-- pitdelay.bas ------------------------------------------------ >>fst'01<< -
' - Erzeugen eines Delays mit einer Aufloesung ab ca. 1 Mikrosekunde durch
' - Programmierung des Programmierbaren-Intervall-Timers (PIT) 8253/8254.
' - Maximale Delayzeit knapp 55 Millisekunden.
' - Unabhaengig von der Rechnergeschwindigkeit.
' - Drei SUBs koennen alternativ benutzt werden:
'  -> Mdelay8    verzoegert in 1-255 Schritten von je 214,55 Mikrosek.
'  -> MDelay16   verzoegert in 1-65535 Schritten von je 0,8381 Mikrosek.
'  -> MDelayASM  wie MDelay16, nur in Assembler ber CALL ABSOLUTE (schnell!)
' - An die SUBs wird ein Zaehler-Zielwert uebergeben, bei dessen Erreichen
'   der Delay beendet wird.
'   - ACHTUNG: Die Zaehler zaehlen abwaerts (65535-0 bzw. 255-0) !
' - Public Domain
'-- V 20010923 ------------------------------ http://home.arcor.de/steini63 -

SCREEN 2    'CGA-Grafik 640x200; auch fuer Palmtops geeignet
LOCATE 2
PRINT "------------------------------------------------------------------------------"
PRINT "Mikro-/Millisekunden-Delay mit PIT 8253/8254 Zaehler 2              >>fst'01<<"
PRINT "------------------------------------------------------------------------------"

LOCATE 6, 1
PRINT "Bitte einen Verzoegerungswert pro Pixel in Millisek. eingeben (2-9; 1=10) >";
DO
 Millisek% = VAL(INKEY$)
LOOP UNTIL Millisek% > 0
IF Millisek% = 1 THEN Millisek% = 10
'Millisek% = 50
PRINT Millisek%

LOCATE 8, 1
PRINT "Schreibe 3 x 640 Pixel mit einem Delay von je"; Millisek%; "Millisekunden ...";

'- 8253/8254 Zielwerte berechnen --------------------------------------------
'8253-Timer 8-Bit-Zielwert:
'Pro Millisekunde werden 4.66086 Tics gezhlt (256 / 54,92549)
Ziel8% = 255 - (Millisek% * 4.66086)

'8253-Timer 16-Bit-Zielwert:
'Pro Millisekunde werden 1193,18 Tics gezhlt (65536 / 54,92549 )
Ziel16& = 65535 - (Millisek% * 1193.18)

'- WICHTIG BEI ZAEHLER 2: Lautsprecher ueber Port 61h, Bit1 aktivieren ------
OUT (&H61), INP(&H61) OR 1

'- 8-Bit Basic-Delay --------------------------------------------------------
LOCATE 12, 1
PRINT USING " 8-Bit-Delay PIT-Timer2 (Aufloesung ###.## Mikrosekunden):"; 54925.49 / 256;
T1 = TIMER              'Startzeit festhalten
FOR i% = 0 TO 639       '640 Schleifen
 PSET (i%, 99)          'Pixel zeichnen
 CALL MDelay8(Ziel8%)   'Delay erzeugen
NEXT
LOCATE 14, 1: PRINT TIMER - T1; " Sekunden"  'Zeitverbrauch anzeigen

'- 16-Bit Basic-Delay -------------------------------------------------------
LOCATE 16, 1
PRINT USING "16-Bit-Delay PIT-Timer2 (Aufloesung #.#### Mikrosekunden):"; 54925.49 / 65536;
T1 = TIMER              'Startzeit festhalten
FOR i% = 0 TO 639       '640 Schleifen
 PSET (i%, 131)         'Pixel zeichnen
 CALL MDelay16(Ziel16&) 'Delay erzeugen
NEXT
LOCATE 18, 1: PRINT TIMER - T1; " Sekunden"  'Zeitverbrauch anzeigen

'- 16-Bit Maschinencode-Delay -----------------------------------------------
LOCATE 20, 1
PRINT "16-Bit-Delay PIT-Timer2 (Maschinencode-Routine):"
'Den Maschinencode in den String "asm$" laden:
CALL MdelayASM(Ziel16&, asm$)
T1 = TIMER                  'Startzeit festhalten
DEF SEG = VARSEG(asm$)      'Das Speicher-Segment von asm$ festlegen
Offset% = SADD(asm$)        'den Speicher-Offset vom asm$ speichern
FOR i% = 0 TO 639           '640 Schleifen
 PSET (i%, 163)             'Pixel zeichnen
 CALL ABSOLUTE(Offset%)     'Delay erzeugen (ASM-Routine aufrufen)
 'Fehler "Unterprogramm nicht definiert" ? -> QuickBasic mit /L starten!
NEXT
DEF SEG                     'Standart Segment wiederherstellen
LOCATE 22, 1: PRINT TIMER - T1; " Sekunden"  'Zeitverbrauch anzeigen
PRINT : PRINT "Taste ...";

'- ENDE ---------------------------------------------------------------------
SLEEP           'Tastendruck abwarten
SCREEN 0        'zurueck zum Textmodus
END             'E n d e

SUB MDelay16 (Ziel&)
OUT (&H43), 180
OUT (&H42), 255
OUT (&H42), 255
OUT (&H42), 255
DO
OUT (&H43), 128
a& = INP(&H42)
b& = INP(&H42)
t& = a& + b& * 256
LOOP UNTIL t& <= Ziel&
EXIT SUB
'Kommentare separat, weil sie die Ausfhrung im Interpretermodus verlangsamen
'ebenso Leerzeilen/Leerzeichen. Folgendes passiert in den vorstehenden Zeilen
'180 = &b10110100 8253/Zaehler2 auf Modus2 setzen
'10=Zaehler2 11=niederwert. Byte, dann hoeherwert. 010=Modus2 0=Binaerzaehler
'Startwert LowByte setzen
'Startwert HighByte setzen
'noch ein OUT (bei schnellen Prozessoren erforderlich, keine Ahnung warum !?)
'Schleife starten
'Werte 8253-intern speichern (Counter Latch)
'LowByte lesen
'HighByte lesen
'Bytes nach Wertigkeit summieren
'Schleife wiederholen, bis Zielwert erreicht
'SUB verlassen
END SUB

SUB MDelay8 (Ziel%)
OUT (&H43), 164
OUT (&H42), 255
OUT (&H42), 255
DO
 x% = INP(&H42)
LOOP UNTIL x% <= Ziel%
EXIT SUB
'Kommentare separat, weil sie die Ausfhrung im Interpretermodus verlangsamen
'ebenso Leerzeilen/Leerzeichen. Folgendes passiert in den vorstehenden Zeilen
'164 = &b10100100 8253/Timer2 auf Modus2 setzen, nur HighByte schreiben/lesen
'10=Zaehler2 10=nur hoeherwertiges Byte 010=Modus2 0=binaerer Wert
'Startwert HighByte setzen )  beste Genauigkeit bei dieser Wiederholung der
'Startwert HighByte setzen )   OUT-Anweisungen bei schnellen Prozessoren (!?)
'Schleife starten
'HighByte lesen
'Schleife wiederholen, bis Zielwert erreicht
'SUB verlassen
END SUB

SUB MdelayASM (Ziel&, asm$)

Lo% = Ziel& MOD 256            'LowByte aus Ziel& ermitteln
Hi% = Ziel& \ 256              'HighByte von Ziel& ermitteln
Ziel$ = CHR$(Lo%) + CHR$(Hi%)  'zu einem String zusammensetzen
                               '(INTEL-Schreibweise: LowByte links)

' ------ Created with Absolute Assembly 2.1 by Petter Holmberg, -97. ------- '
asm$ = ""
asm$ = asm$ + CHR$(&HE4) + CHR$(&H61)  ' IN AL,61       ;Port 61h lesen
asm$ = asm$ + CHR$(&HC) + CHR$(&H1)    ' OR AL,01       ;Bit1 setzen (Laustspr. an)
asm$ = asm$ + CHR$(&HE6) + CHR$(&H61)  ' OUT 61,AL      ;und nach Port 61h ausgeben
asm$ = asm$ + CHR$(&HB0) + CHR$(&HB4)  ' MOV AL,B4      ;AL mit 180 (B4h) laden
asm$ = asm$ + CHR$(&HE6) + CHR$(&H43)  ' OUT 43,AL      ;und auf Port 43h ausgeben
asm$ = asm$ + CHR$(&HB0) + CHR$(&HFF)  ' MOV AL,FF      ;AL mit 255 laden (FFh)
asm$ = asm$ + CHR$(&HE6) + CHR$(&H42)  ' OUT 42,AL      ;und auf Port 42h ausgeben (LowByte)
asm$ = asm$ + CHR$(&HE6) + CHR$(&H42)  ' OUT 42,AL      ;und auf Port 42h ausgeben (HighByte)
asm$ = asm$ + CHR$(&HE6) + CHR$(&H42)  ' OUT 42,AL      ;nochmal fr schnelle Prozessoren
asm$ = asm$ + CHR$(&HB0) + CHR$(&H80)  ' Pit: MOV AL,80 ;Marke "Pit";AL mit 128 (80h) laden
asm$ = asm$ + CHR$(&HE6) + CHR$(&H43)  ' OUT 43,AL      ;und auf Port 43h ausgeben (CounterLatch)
asm$ = asm$ + CHR$(&HE4) + CHR$(&H42)  ' IN AL,42       ;Byte von Port 42h nach AL lesen (LowByte)
asm$ = asm$ + CHR$(&H88) + CHR$(&HC4)  ' MOV AH,AL      ;und nach AH schieben
asm$ = asm$ + CHR$(&HE4) + CHR$(&H42)  ' IN AL,42       ;Byte von Port 42h nach AL lesen (HighByte)
asm$ = asm$ + CHR$(&H86) + CHR$(&HE0)  ' XCHG AH,AL     ;AH/AL tauschen, AX hat den 16-Bit Wert
asm$ = asm$ + CHR$(&H3D) + Ziel$       ' CMP AX,(Ziel)  ;AX mit Zielwert vergleichen
asm$ = asm$ + CHR$(&H77) + CHR$(&HEF)  ' JA Pit         ;wenn groesser (Ziel noch nicht erreicht) nach Pit springen
asm$ = asm$ + CHR$(&HCB)               ' RETF           ;sonst zurueck nach Basic
' ------ Created with Absolute Assembly 2.1 by Petter Holmberg, -97. ------- '
END SUB

